// DJ -- A printing utility for the HP DeskJet.  Modified by Ron Abramson,
//			 CIS ID 74676,3345, from the 'LJ' program for the LaserJet.
//
// This program prints files 2-up on the HP DeskJet Plus printer.  The
// files are printed in a "landscape" font at 20 characters to the inch.
// Optional arguments may be entered from the command line as well as from a
// full-screen initialization mode.
//
// Usage (from the command line) is: DJ [flags] file1 [file2 file3 ...]
//
// Where file# is a valid MS-DOS filename.  Wildcard filenames may be
// entered from the command line.
//
// Originally written [as LJ] for the HP LaserJet by Joe Barnhart and
// subsequently modifed by Ray Duncan and Chip Rabinowitz.  See .doc file
// for full history.  This program has been placed in the public domain
// for use without profit.
//
// Page layout:
//  Horizontal, 10 1/3 inches printable, 207 columns at 20 cpi
//   1/2 inch at left edge of landscape page is physically non-printable
//   col 0   - 1st text col, 1/2 inch from left landscape edge of paper
//   col 96  - last text col on left half of page, allows width of 97 chars
//   col 100 - vertical line separating 2 halves of page
//   col 110 - 1st col of right half text
//   col 206 - last text col on left half of page, allows width of 97 chars
//   1/6 inch at right edge is physically non-printable
//  Vertical, 8 inches printable, 76 lines at 5/48 inch per line
//   row 0-2   blank (we are setting DeskJet top mar = 3)
//   row 3-4   header
//   row 5-70  text (text lines 1-66)
//   row 71-75 effectively a bottom margin
//------------------------------------------------------------------------------

#include <stdio.h>
#include <time.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <process.h>

#include "atget.h" // header for screen editing module

int main( int, char** );
void printfile( char* );
int printpage( void );
void header( char*, int );
void dovert( void );
void copyname( char*, char* );
void usage( void );
void timestamp( char* );
void datestamp( char* );

#define MAXVERT 71	// highest possible text row, given our margins
int maxline = 66;		// text lines per page
int maxcol = 97;		// text width

FILE *lp;						// Line Printer (actual or virtual)
FILE *fp;           // FILE pointer for the file to be printed

// integer flags and corresponding strings (for editing)...
int hdrflag = 0; char hdrstg[2] = " "; // 1 = suppress headers
int pgflag = 0;	char pgstg[2] = " "; // 1 = new file doesn't start new sheet
int tab = 4; char tabstg[2] = "4"; // default horiz tabs = 4 spaces
int book = 0;	char bookstg[2] = " "; // 1 = indent for book format
int numbers = 0; char numberstg[2] = " "; // 1 = do line numbering
int formfeed = 0; char formfeedstg[2] = " "; // 1 = ignore form feeds in file
int draft = 0; char draftstg[2] = " "; // 1 = draft mode
int bidir = 1; char bidirstg[2] = "x"; // (bi-directional = default)
int fileout = 0; char fileoutstg[2] = " "; // 1 = redirect to file dj.out
int duplex = 0; char duplexstg[2] = " "; // 1 = two-sided printing
char maxlinestg[3] = "66"; // string version of maxlines (above)

int pgside = 0; // left/right half of page indicator
int do_print = 1; // active print toggle - for duplex
int line_no = 1;
int pagenum;

// main()
//------------------------------------------------------------------------------

int main(int argc, char *argv[])
{
	int filenum;
	char fname[70];
	int i;
	int lastkey; // result code from edit operation

	// get flags (if any) from command line
	// and set corresponding strings appropriately...
	for (i = 1; i < argc; i++)  {
		if (*argv[i] != '/' && *argv[i] != '-') // check for flags
			break; // and exit if none
		argv[i]++;
		switch (tolower(*argv[i]))  {
			case '?':
				usage();
				exit ( 1 );
			case 'h':
				hdrflag = 1;
				strcpy( hdrstg, "x" );
				break;
			case 't':
				tab = atoi(++argv[i]);
				itoa( tab, tabstg, 10 );
				break;
			case 'l':
				maxline = atoi(++argv[i]);
				itoa( maxline, maxlinestg, 10 );
				break;
			case 'p':
				pgflag = 1;
				strcpy( pgstg, "x" );
				break;
			case 'n':
				numbers = 1;
				strcpy( numberstg, "x" );
				break;
			case 'b':
				book = 1;
				strcpy( bookstg, "x" );
				break;
			case 'd':
				duplex = 1;
				strcpy( duplexstg, "x" );
				break;
			case 'i':
				formfeed = 1;
				strcpy( formfeedstg, "x" );
				break;
			case 'r':
				draft = 1;
				strcpy( draftstg, "x" );
				break;
			case 'u':
				bidir = 0;
				strcpy( bidirstg, " " );
				break;
			case 'f':
				fileout = 1;
				strcpy( fileoutstg, "x" );
				break;
			default:
				gotoxy( 1, 18 );
				printf( "\aInvalid Flag Specified \('%s'\) -- ignored...\n", --argv[i] );
				break;
		} //switch ...
	} //for ...

	// next, do full-screen editing on the parameters...
	gt.gc = 0;		// initialize screen editing
	beep_on = 0;
	cls();
	strset( fname, 0 ); // get 1st filename from cmd line, if any
	if ( i < argc ) strcpy( fname, argv[i] );
	rpad( fname, 69 ); // pad strings for editing
	rpad( maxlinestg, 2 );

	atsay( 1, 1, "DeskJet Printing Facility");
	atsay( 3, 1, "FILE:" );
	atget( 3, 7, fname );
	atsay( 5, 1, "x     Item...");
	atsay( 6, 7, "Suppress page headers" );
	atget( 6, 1, hdrstg );
	atsay( 7, 7, "Set tab stop value of 1 to 8" );
	atget( 7, 1, tabstg );
	atsay( 8, 7, "Do not leave blank half pages between files" );
	atget( 8, 1, pgstg );
	atsay( 9, 7, "Print line numbers" );
	atget( 9, 1, numberstg );
	atsay(10, 7, "Indent for punching \'book style\'" );
	atget(10, 1, bookstg );
	atsay(11, 7, "Duplex (two-sided) printing" );
	atget(11, 1, duplexstg );
	atsay(12, 7, "Set lines/page from 1 to 66");
	atget(12, 1, maxlinestg );
	atsay(13, 7, "Ignore formfeed characters" );
	atget(13, 1, formfeedstg );
	atsay(14, 7, "Redirect output to file dj.out" );
	atget(14, 1, fileoutstg);
	atsay(15, 7, "Draft mode printing (for speed)" );
	atget(15, 1, draftstg);
	atsay(16, 7, "Bi-directional printing (for speed)" );
	atget(16, 1, bidirstg);

	lastkey = readgets();
	gotoxy( 1, 18 );
	if ( lastkey == 4 ) {
		printf( "\aJob cancelled.\n" );
		exit ( 1 );
	}
	// process results of cmd line and full-screen input...
	rtrim( fname );
	ltrim( fname );
	if ( fname[0] == 0 ) {
		printf( "\aNo file specified\n" );
		exit (1);
	}

	if (hdrstg[0] != 32) {
		hdrflag = 1;
		if ( MAXVERT - maxline >= 2 ) maxline += 2;	// add lines where hdr was
	}
	if (tabstg[0] != '4') {
		tab = atoi( tabstg );
		if (tab < 1 || tab > 8)  {
			printf("Invalid tab value specified -- defaulting to 4\n");
			tab = 4;
		} else {
			//printf( "Tabs set to %d\n", tab );
		}
	}
	if ( ( maxline = atoi(maxlinestg) ) != 66 ) {
		if ( maxline < 1 || maxline > (MAXVERT - !hdrflag*2) )  {
			printf("Invalid lines/page value specified -- defaulting to 66\n");
			maxline = 66;
		} else {
			//printf( "Lines per page set to %d\n",maxline );
		}
	}
	if (pgstg[0] != 32) {
		pgflag = 1;
	}
	if (numberstg[0] != 32) {
		numbers = 1;
	}
	if (bookstg[0] != 32) {
		book = 1;
		maxcol = 89;
	}
	if(duplexstg[0] != 32) {
		duplex = 1;
		if((argc - 1) > i) {
			printf("\aOnly one file at a time may be printed when using DUPLEX mode.\n");
			exit ( 1 );
		} else {
			//printf( "Duplex \(two-sided\) printing selected\n" );
		}
	}
	if (formfeedstg[0] != 32 ) {
		formfeed = 1;
	}
	if (draftstg[0] != 32) {
		draft = 1;
	}
	if (bidirstg[0] == 32) {
		bidir = 0;
	}
	if (fileoutstg[0] != 32) {
		fileout = 1;
	}

  // open output stream
	_fmode = O_TEXT;
	if ( fileout )
		lp = fopen("dj.out","wt");			// file output for testing
	else
		lp = freopen("PRN","w",stdprn);	// print output

	/* initialize the DeskJet for landscape printing */
	//key=>       reset lscp noperf 20 cpi/6pt vmi=5 txlen 70 tp mar 3
	fprintf( lp,"\033E\033&l1o0L\033\(s20h6V\033&l5C\033&l70F\033&l3E" );
	if ( draft ) fprintf( lp, "\033(s1Q" ); // set draft mode
	if ( bidir ) fprintf( lp, "\033&k1W" ); // set bi-dir print

	// process file list (or single file)
	if ( (argc - 1 ) > i ) {
		for(filenum = i; filenum < argc; filenum++ ) {
			copyname(fname, argv[filenum]);
			fp = fopen(fname ,"r");
			if( fp == NULL ) {
				printf( "\n\aFile %s not found.\n", fname );
			} else {
				printf( "\nNow printing %s\n", fname );
				printfile( fname );
				fclose( fp );
			}
		}
	} else {
		fp = fopen( fname, "r" );
		if ( fp == NULL ) {
			printf( "\n\aFile %s not found.\n", fname );
		} else {
			printf( "Now printing %s\n", fname );
			printfile( fname );
			fclose( fp );
		}
	}

	fprintf( lp, "\r\033E" );      // clear DeskJet
	fclose(lp);

	return filenum;
}

// printfile (filename)
//------------------------------------------------------------------------------
void printfile( char *filename ) {

	int retval = 0;
	int printed = 0;
	int pass = 0;
	char c2;
	int f_feed = 0;
	int phys_pages = 0;
	int plus = 1;

	pagenum = 1;
do_it_again:
	while( (feof(fp) == 0) && (retval == 0) ) {
		f_feed = 0;
		if ( pgside == 0 ) {
			printed = 0;
			c2 = fgetc(fp);
			if( feof(fp) ) break;
			switch (c2) {
				case EOF:
				case 0x1a:
					retval = -1;
					break;
				default:
					printed = 1;
					// following printer strings set margins...
					if (book && do_print)
						fprintf(lp, "\033&a0r0c96m7L\r"); //set to left half
					else if (do_print)
						fprintf(lp, "\033&a0r0c96m0L\r"); //set to left half
					ungetc(c2,fp);
					header( filename, pagenum++ );      // title top of page
					retval = printpage();               // print one page
					pgside ^= 1;
			}// end switch
		}// end if (pgside == 0)

		if (do_print) dovert(); // lay vertical stripe now

		if(feof(fp) == 0 && retval == 0) {        // if more to print...
			// following printer strings set margins...
			if (book && do_print)
				fprintf(lp, "\033&a0r0c206m117L\r");  // DeskJet to right half
			else if (do_print)
				fprintf(lp, "\033&a0r0c206m110L\r");  // DeskJet to right half
			c2 = fgetc(fp);
			if ( feof(fp) ) break;
			switch (c2) {
				case EOF:
				case 0x1a:
					retval = -1;
					break;
				default:
					ungetc(c2,fp);
					header(filename, pagenum++);          // title top of page
					retval = printpage();                 // print one page
					pgside ^= 1;
			}//end switch
		}// end if (feof...)
		if (pgside == 0 && printed && do_print) {
			fputc('\f', lp);
			f_feed = 1;
		}
		else if (pgflag == 0 && printed && do_print) {
			fputc('\f', lp);
			pgside = 0;
			f_feed = 1;
		}
		if ( f_feed ) {
			if(plus)
				++phys_pages;
			else
				--phys_pages;
		}
		if(duplex) do_print ^= 1;
	} //end while

	if ( duplex ) {
		if ( pass ) {
			while ( phys_pages ) {
				fputc( '\f',lp );
				--phys_pages;
			}
		return;
		}
		plus = 0;
		if( !f_feed && retval == 0 ) fputc( '\f',lp );
		fflush(lp);
		++pass;
		rewind(fp);
		retval = 0;
		pagenum = 1;
		do_print = 0;
		printf("\nReinsert the paper and press any key when ready\n");
		bioskey(0);
		goto do_it_again;
	}
}

// printpage - print a logical page
//------------------------------------------------------------------------------
int printpage( void )  {
	char c;
	int line, col;
	static int cont = 0;
	static char *cont1 = "---->";

	line = col = 0;
	if(cont) {
		if(do_print) fprintf(lp,cont1);
		col = strlen(cont1);
		cont = 0;
	}
	while( line < maxline ) {
		c = fgetc(fp);
		if(feof(fp)) return(-1);
		if(col>maxcol) {
			line++;
			switch(c) {
				case '\n':
				case '\r':
				case '\f':
				case EOF:
				case 0x1a:
					break;
				default:
					if(line >= maxline) {
						cont = 1;
						ungetc(c,fp);
						return(0);
					}
					if(do_print) fprintf(lp,"\n%s",cont1);
					col = strlen(cont1);
					break;
			} // switch...
		} //if...
		if(col == 0) {
			if(do_print) {
				printf("Printing page %4.4d line %4.4d\r",pagenum-1,line);
			} else {
				printf("Skipping page %4.4d line %4.4d\r",pagenum-1,line);
			}
			if(numbers) {
				if(do_print) fprintf(lp,"%4.4d: ",line_no);
				col = 6;
			}
			++line_no;
		}

		switch(c) {
			case '\n':
				col = 0;
				line++;
				if( line < maxline )
					if(do_print) fprintf(lp,"\n");
				break;
			case '\r':	// ignore lone CR
				break;
			case '\t':	// expand tabs
				do
					if(do_print) fputc(' ',lp);
				while ( (++col % tab) != 0 );
				break;
			case '\f':							// Page break
				if(formfeed) break;		// ignore, or
				if(line != 0)
					line = maxline;			// force termination of loop
				break;
			case EOF:								// EOF mark
			case 0x1a:							// EOF mark
				return(-1);
			default:
				if(do_print) fputc(c,lp); // default: print the character
				col++;
				break;
		}
	}// while...
	return(0);
}

// header - print a page header
//------------------------------------------------------------------------------
void header( char *filename, int pagenum ) {
	char datestr[11], timestr[11];
	if (hdrflag)  {
		return;		// skip if flag set
	}
	timestamp(timestr);
	datestamp(datestr);
	if(do_print) fprintf(lp,"\033(s3B"); // bold on
	if(book && do_print)
		fprintf(lp, \
			"File: %-46s%s   %s  --  Page: %03d\n\n", \
			filename,datestr,timestr,pagenum);
	else if(do_print)
		fprintf(lp, \
			"File: %-53s%s   %s  --  Page: %03d\n\n", \
			filename,datestr,timestr,pagenum);
	if(do_print) fprintf(lp,"\033(s0B");  // bold off
}

// timestamp()
//------------------------------------------------------------------------------
void timestamp( char *timestr ) {
	struct tm *tod;
	time_t tt;
	tt = time(NULL);
	tod = localtime(&tt);
	sprintf(timestr,"%02d:%02d", tod->tm_hour, tod->tm_min);
	return;
}

// datestamp()
//------------------------------------------------------------------------------
void datestamp( char *datestr ) {
	struct tm *tod;
	time_t tt;
	tt = time(NULL);
	tod = localtime(&tt);
	sprintf(datestr,"%02d/%02d/%02d", tod->tm_mon + 1, tod->tm_mday,
	tod->tm_year);
	return;
}

// dovert() - draw a vertical line down the center of the physical page
//------------------------------------------------------------------------------
void dovert( void ) {
	int line = 1;

	fprintf(lp,"\033&a0r0c102m100L\r|");
	while(line++ < ( maxline + !hdrflag*2 + 1 ) ) fprintf(lp,"\n|");
}

// copyname - copy a file name converting to upper case.
//------------------------------------------------------------------------------
void copyname( char *to, char *from ) {
	while (*from)
		*to++ = toupper(*from++);
	*to = 0;
}

// usage()
//------------------------------------------------------------------------------
void usage( void ) {
	printf("\n\aUSAGE:\n\n");
	printf("C:\\> DJ [flags] filename1 [filename2 ... ]\n");
	printf("\n\tItems shown in brackets are optional.");
	printf("\n\tFilename(s) may include wildcards.");
	printf("\n\n\tFlags are:\n");
	printf("\t\t-h   (or /h)   suppress page headers\n");
	printf("\t\t-tx  (or /tx)  set tab stop value (where x is 1 to 8)\n");
	printf("\t\t-p   (or /p)   do not leave blank half pages between files\n");
	printf("\t\t-n   (or /n)   print line numbers\n");
	printf("\t\t-b   (or /b)   indent for punching \'book style\'\n");
	printf("\t\t-d   (or /d)   for duplex (two-sided) printing\n");
	printf("\t\t-lxx (or /lxx) set lines/page to x (where x is 1 to 66)\n");
	printf("\t\t-i   (or /i)   ignore formfeed characters\n");
	printf("\t\t-r   (or /r)   draft mode printing (for speed)\n");
	printf("\t\t-u   (or /u)   uni-directional printing (more crisp)\n");
	printf("\t\t-f   (or /f)   redirect output to file dj.out\n");
	printf("\nExample: C:\\> DJ -t2 -n dj.doc <Enter>\n");
}

//------------------------------------------------------------------------------
//eof dj*.c
